#include "chat-gui.h"
#include <iostream>
#include <string>
#include <thread>
#include <mutex>
#include "UniSocket.hpp"
using namespace std;
    
UserInterface ui;
UniSocket * sock;
Fl_Text_Buffer * buf;
    
mutex mtx;
bool tryClose;
    
// Thread, der dauerhaft auf eingehende Daten wartet
void doReceive(void) {
	while(true) {
		string msg = sock->recv();
		mtx.lock();
		if(msg == "[bye]" && tryClose) {
			mtx.unlock();
			break;
		}
		mtx.unlock();
		msg += "\n";
		buf->append(msg.c_str());
		// zeichnet GUI-Elemente neu, falls Inhalte neu sind
		Fl::check();
	}
	ui.connectBtn->label("connect");
	sock->close();
}
    
// wird beim Klick auf den connect- bzw. disconnect-Button aufgerufen
void doConnect(Fl_Widget * obj, void *) {
	string lbl = obj->label();
	if(lbl == "connect") {
		try {
			sock = new UniSocket(
				ui.ipInput->value(),
				ui.portInput->value(),
				"[ENDE]"
			);
			mtx.lock();
			tryClose = false;
			mtx.unlock();
			thread t(doReceive);
			t.detach();
			
			// bei "connect" passende GUI-Elemente ein/ausschalten
			ui.msgInput->activate();
			ui.sendBtn->activate();
			ui.ipInput->deactivate();
			ui.portInput->deactivate();
			// der Connect-Button wird umbenannt
			obj->label("disconnect");
		} catch(UniSocketException e) {
			e._msg += "\n";
			cout << e._msg;
			buf->append(e._msg.c_str());
		}
	} else {
		// bei "disconnect" passende GUI-Elemente ein/ausschalten
		ui.msgInput->deactivate();
		ui.sendBtn->deactivate();
		ui.ipInput->activate();
		ui.portInput->activate();
		// Setzen der Markierung fuer Empfangs-Thread, welcher
		// auf ein [bye] vom Server wartet ...
		mtx.lock();
		tryClose = true;
		mtx.unlock();
		// ... nachdem "ich" Server ein [close] gesendet habe:
		sock->send("[close]");
	}
}
    
// wird beim Klick auf den send-Button aufgerufen
void doMsg(Fl_Widget * obj, void *) {
	sock->send(ui.msgInput->value());
	string ret = "me: " + string(ui.msgInput->value()) + "\n";
	buf->append(ret.c_str());
	// Eingabefeld leeren
	ui.msgInput->value("");
}
    
// Erzeugung notwendiger Objekte und Konfiguration 
int main(void) {
	Fl_Double_Window * win = ui.make_window();
	Fl::scheme("gtk+");
	// Die Chat-Log Ausgabe wird mit einem Textbuffer verbunden
	buf = new Fl_Text_Buffer();
	ui.chatLog->buffer(buf);

	tryClose = false;	
	// fuer die Events der Buttons werden 
	// Callback-Funktionen hinterlegt
	ui.connectBtn->callback(doConnect);
	ui.sendBtn->callback(doMsg);

	win->show();
	// Starten der GUI-Endlos-Schleife
	return Fl::run();
}
